/**************************************************************************
***************************************************************************
			PROBLEME DE LAMBERT
	PROGRAMME CONTENANT LES FONCTIONS TRAITANT LE CAS HYPERBOLIQUE
***************************************************************************
**************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include<conio.h>



#define pi 3.14




//DECLARATION DES FONCTIONS EXTERNES UTILES A CE FICHIER:

extern double prodscal(double[],double[]);
extern double norme(double[]);
extern double TempsHyperbole(double,double);
extern double asinh(double);
extern double acosh(double);

extern void somme(double[],double[],double[]);
extern void prodvect(double[],double[],double[]);
extern void unitaire(double[],double[]);
extern void multiplication_scalaire(double,double[],double[]);




//DECLARATION DES VARIABLES EXTERNES UTILES A CE FICHIER:

extern double Delta_T;    /* temps entre par l'utilisateur */
extern double Lambda;
extern double Mu;
extern double W[4];
extern double RayonVecteurDepart[4],RayonVecteurArrivee[4];
extern double R1,R2;
extern double Delta;  /* angle entre les vecteurs de depart et d'arrivee */
extern double S;
extern double D;
extern double AxeI[4];
extern double AxeK[4];
extern double AxeJ[4];
extern double DureeMini;
extern double Angle;

extern int Helios;
extern int NTour;

extern char nom_mission[20];
extern char azerty;




//DECLARATION DES VARIABLES LOCALES A CE FICHIERS:

double inclinaisonh;
double excentriciteh;
double ah;            /* demi grand axe de l'hyperbole */
double phi1h,phi2h;   /* anomalie excentrique */
double teta1h,teta2h; /* anomalie vraie aux points de depart et d'arrivee */
double alphah;
double longitudeh;    /* longitude vernale */
double argumenth;     /* argument nodale au perigee */
double Sol_O_Tourh;   /* alpha calcule */
double vitesse1h[4],vitesse2h[4]; /* vitesse au depart et a l'arrivee */
double AxeWh[4];
double Nvitesse1h,Nvitesse2h; /* norme des vitesses */

int cas_de_volh;




/************************************************************************/
/**************** solution equation hyperbolique ************************/
/************************************************************************/


void solution_equation_hyperbolique()
{
	double Temps_Reduit;
	double Xmax,Xmin,dx,produit_test;
	double X[12],F[12];
	int boolean;


	/* initialisation des variables */
	Temps_Reduit = (Delta_T*86400)/(sqrt(exp(3*log(S))/8/Mu));
	Xmax = 5;
	Xmin = 0.0001;
	boolean = 1;


	/* definition de l'intervalle [Xmin,Xmax]
	   Seul varie la borne superieur car un vol hyperbolique
	   est toujours possible */

	while (boolean!=0)
	{
		if (TempsHyperbole(Xmax,Lambda)>=Temps_Reduit)
		       (Xmax = Xmax+0.1);
		if(TempsHyperbole(Xmax,Lambda)<Temps_Reduit)
		       (boolean = 0);
	}


	/* determination de alpha par comparaisons successives entre
	   le temps theorique et celui souhaite par l'utilisateur */
	boolean = 1;

	while (boolean!=0)
	{

		dx = (Xmax-Xmin)/10;  /* longueur du pas */


		for (int i=1;i<=11;i++)
		{
			X[i] = Xmin+(i-1)*dx;
			F[i] = TempsHyperbole(X[i],Lambda);
		}

		for (int j=1;j<11;j++)
		{
			produit_test = (Temps_Reduit-F[j])*
					(Temps_Reduit-F[j+1]);
			if (produit_test<=0)
			{
				Xmin = X[j];
				Xmax = X[j+1];
			}

		}

		/* On affecte la valeur de alpha lorsque l'on estime avoir
		   atteint la precision souhaitee */
		if ((Xmax-Xmin)<1e-7)
		{
			Sol_O_Tourh = (Xmax+Xmin)/2;
			boolean = 0;
		}
	}
}





/***************************************************************************/
/********************* parametres orbitaux de l'hyperbole ******************/
/***************************************************************************/

void parametres_principaux_hyperbole()
{

	double beta;
	double psi,phi;
	double phi11,phi12,phi21,phi22;


	/* calcul de phi et de psi */
	alphah = Sol_O_Tourh;
	beta = 2*asinh(Lambda*sinh(alphah/2));

	phi = (alphah+beta)/2;
	psi = (alphah-beta)/2;


	/* excentricite */
	excentriciteh = sqrt((cosh(phi)*cosh(phi))-((R1-R2)*sinh(phi)/D)
						   *((R1-R2)*sinh(phi)/D));

	/* demi-grand axe */
	ah = S/(cosh(alphah)-1);


	/* calcul de phi1 et de phi2 */
	phi11 = acosh((1+R1/ah)/excentriciteh);
	phi12 = -phi11;
	phi21 = acosh((1+R2/ah)/excentriciteh);
	phi22 = -phi21;

	/* On verifie l'orientation des angles phi1 et phi2 avant
	   affectations. Les incertitudes sont dues a l'utilisation
	   d'un acos. */
	if (fabs(phi21-phi11-2*psi)<1e-4)
	{
		phi1h = phi11;
		phi2h = phi21;
	}

	if (fabs(phi21-phi12-2*psi)<1e-4)
	{
		phi1h = phi12;
		phi2h = phi21;
	}

	if (fabs(phi22-phi11-2*psi)<1e-4)
	{
		phi1h = phi11;
		phi2h = phi22;
	}

	if (fabs(phi22-phi12-2*psi)<1e-4)
	{
		phi1h = phi12;
		phi2h = phi22;
	}

}




/**************************************************************************/
/****************** parametres orbitaux de l'hyperbole ********************/
/**************************************************************************/


void parametres_hyperbole()
{
	double Radial1[4],Orthoradial1[4];
	double Radial2[4],Orthoradial2[4];
	double VectExc[4],VectNod[4];
	double CV1[4],CV2[4],CV3[4],CV4[4],CV5[4];
	double constante;
	double Cteta1,Cteta2,Steta1,Steta2;
	double tan1,tan2;

	/* Calcul de l'anomalie vraie */

	tan1 = sqrt((excentriciteh+1)/(excentriciteh-1))*
		sinh(phi1h/2)/cosh(phi1h/2);
	/*
	Cteta1 = (cosh(phi1h)-excentriciteh)/(1-excentriciteh*cosh(phi1h));
	Cteta2 = (cosh(phi2h)-excentriciteh)/(1-excentriciteh*cosh(phi2h));
	Steta1 = ah/R1*sqrt(excentriciteh*excentriciteh-1)*sinh(phi1h);
	Steta2 = ah/R2*sqrt(excentriciteh*excentriciteh-1)*sinh(phi2h);
	*/
	Cteta1 = (1-tan1*tan1)/(1+tan1*tan1);
	Steta1 = 2*tan1/(1+tan1*tan1);

	tan2 = sqrt((excentriciteh+1)/(excentriciteh-1))*
		sinh(phi2h/2)/cosh(phi2h/2);
	Cteta2 = (1-tan2*tan2)/(1+tan2*tan2);
	Steta2 = 2*tan2/(1+tan2*tan2);

	if (Cteta1>=0)
	teta1h=acos(Cteta1);
	else teta1h=-acos(Cteta1);


	/******* vitesse 1 (au depart) *******/
	constante = sqrt(Mu/ah/(excentriciteh*excentriciteh-1));
	unitaire(W,AxeWh);
	unitaire(RayonVecteurDepart,Radial1);
	prodvect(AxeWh,Radial1,Orthoradial1);
	multiplication_scalaire(constante*excentriciteh*Steta1,Radial1,CV2);
	multiplication_scalaire(constante*(1+excentriciteh*Cteta1),Orthoradial1,CV3);
	somme(CV2,CV3,vitesse1h);
	Nvitesse1h = norme(vitesse1h);


	Cteta1 = (cosh(phi1h)-excentriciteh)/(1-excentriciteh*cosh(phi1h));
	Cteta2 = (cosh(phi2h)-excentriciteh)/(1-excentriciteh*cosh(phi2h));
	Steta1 = ah/R1*sqrt(excentriciteh*excentriciteh-1)*sinh(phi1h);
	Steta2 = ah/R2*sqrt(excentriciteh*excentriciteh-1)*sinh(phi2h);

	if (Cteta2>=0)
	teta2h=acos(Cteta2);
	else teta2h=-acos(Cteta2);

	/******* vitesse 2 (a l'arrivee) *******/
	constante = sqrt(Mu/ah/(excentriciteh*excentriciteh-1));
	unitaire(RayonVecteurArrivee,Radial2);
	prodvect(AxeWh,Radial2,Orthoradial2);
	multiplication_scalaire(constante*excentriciteh*Steta2,Radial2,CV2);
	multiplication_scalaire(constante*(1+excentriciteh*Cteta2),Orthoradial2,CV3);
	somme(CV2,CV3,vitesse2h);
	Nvitesse2h = norme(vitesse2h);

	/* vecteur excentricite */
	prodvect(RayonVecteurDepart,vitesse1h,CV2);
	prodvect(vitesse1h,CV2,CV3);
	multiplication_scalaire(1/Mu,CV3,CV4);
	unitaire(RayonVecteurDepart,CV5);
	somme(CV4,CV5,VectExc);


	/* vecteur nodal */
	prodvect(AxeK,AxeWh,VectNod);


	/* Argument du perigee
	   Sa valeur elle aussi calcule a partir d'un arccosinus
	   hyperbolique doit etre controlee */
	double prod;
	prod = prodscal(VectExc,VectNod);
	argumenth = acos(prod/excentriciteh);
	if (VectExc[3]<0) (argumenth = 2*pi-argumenth);


	/* inclinaison */
	/* Si l'inclinaison est trop faible elle sera prise nulle de meme
	   que la longitude et on modifiera le vecteur nodale alors
	   incorrecte. Dans le cas inverse, il faut tout de meme faire
	   attention a la valeur de l'inclinaison calculee grace a un
	   arcosinus hyperbolique */
	inclinaisonh = acos(AxeWh[3]);
	if (fabs(inclinaisonh)<1e-5)
	{
		longitudeh = 0;
		for(int k=1;k<4;k++)
			VectNod[k]=AxeI[k];
	}

	else
	{
		unitaire(VectNod,VectNod);
		longitudeh = acos(VectNod[1]);
		if (VectNod[2]<0) longitudeh = 2*pi-longitudeh;
	}




}



/**************************************************************************/
/********************** classification des cas ****************************/
/**************************************************************************/

int classification_cas_hyperbolique()
{
	int e1,e2,e3,e4,e5;
	/* La position des points de depart et d'arrivee implique
	   differents cas de vol */
	/* Assignation des booleens */
	if(R1<R2) (e1 =1);
	else (e1 = 0);



	if(Lambda>0) (e2 = 1);
	else (e2 = 0);

	if(phi1h>0) (e3 = 1);
	else (e3 = 0);

	if(phi2h>0) (e4 = 1);
	else (e4 =0);


	if((e1==1)&&(e2==1)&&(e3==1)&&(e4==1))
		(cas_de_volh = 11);
	if((e1==1)&&(e2==1)&&(e3==0)&&(e4==1))
		(cas_de_volh = 12);
	if((e1==1)&&(e2==0)&&(e3==0)&&(e4==1))
		(cas_de_volh = 13);
	if((e1==0)&&(e2==1)&&(e3==0)&&(e4==0))
		(cas_de_volh = 14);
	if((e1==0)&&(e2==1)&&(e3==1)&&(e4==1))
		(cas_de_volh = 15);
	if((e1==0)&&(e2==0)&&(e3==0)&&(e4==1))
		(cas_de_volh = 16);

	return cas_de_volh;

}





